package net.hostsharing.hsadminng.hs.hosting.asset.factories;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import net.hostsharing.hsadminng.hs.booking.generated.api.v1.model.HsHostingAssetAutoInsertResource;
import net.hostsharing.hsadminng.hs.booking.item.BookingItemCreatedAppEvent;
import net.hostsharing.hsadminng.mapper.StandardMapper;
import net.hostsharing.hsadminng.persistence.EntityManagerWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

@Component
public class HsBookingItemCreatedListener implements ApplicationListener<BookingItemCreatedAppEvent> {

    @Autowired
    private EntityManagerWrapper emw;

    @Autowired
    private ObjectMapper jsonMapper;

    @Autowired
    private StandardMapper standardMapper;

    @Override
    @SneakyThrows
    public void onApplicationEvent(final BookingItemCreatedAppEvent event) {
        final var newBookingItemRealEntity = event.getEntity().getBookingItem();
        final var asset = jsonMapper.readValue(event.getEntity().getAssetJson(), HsHostingAssetAutoInsertResource.class);
        final var factory = switch (newBookingItemRealEntity.getType()) {
            case PRIVATE_CLOUD, CLOUD_SERVER, MANAGED_SERVER -> null; // for now, no automatic HostingAsset possible
            case MANAGED_WEBSPACE -> null; // TODO.impl: implement ManagedWebspace HostingAsset creation, where possible
            case DOMAIN_SETUP -> new DomainSetupHostingAssetFactory(emw, newBookingItemRealEntity, asset, standardMapper);
        };
        if (factory != null) {
            event.getEntity().setStatusMessage(factory.performSaveProcess());
            emw.persist(event.getEntity()); // TODO.impl: once we implement retry, we might need merge
        }
    }
}
